#GameKitCenter

###Design Objective
A system for managing iOS game achievements and leaderboards. Its functionalities are based on the Game Center. It can be said that GameKitCenter is an "offline" version of the Game Center.

You can choose to have it synchronized with the Game Center (effortlessly, of course), in which case the library simply becomes a cache for storing Game Center data locally.

###Features
- Achievements
- Achievement notification
- Leaderboards

###Basic: Using the Provided Classes


#####Getting Started

GameKitCenter comes with 4 main classes: `GameKitCenter`, `StandardGameKitAchievement`, `StandardGameKitLeaderboard` and `StandardGameKitScore`. You only need to care about the first class.

But before you can use the library, you need to define the achievements and leaderboards inside a plist.

For `StandardGameKitAchievement`, the required keys are as follows:

- Identifier (string)
- Title (string)
- Points (Integer)

For `StandardGameKitLeaderboard`, the required keys are as follows:

- Identifier (string)
- Name (string)
- ScoreFormatSuffixSingular (string)
- ScoreFormatSuffixPlural (string)
- ScoreRange (NSRange: e.g. {0, 10000})
- SortOrder (string: HighToLow or LowToHigh)

Store it inside a plist so that the structure looks like this (or you can check the sample Xcode project for a sample of this plist):

	Root (dictionary)
	
		Achievements (array)
		
			Item 0 (dictionary)
				Identifier 	-> Achievement_1
				Title 		-> Achievement 1
				Points 		-> 10
				
			Item 1 (dictionary)
			  .
			  .
			  .
			  .
			  
		Leaderboards (array)
		
			Item 0 (dictionary)
				Identifier					-> Leaderboard_1
				Name						-> Leaderboard 1
				ScoreFormatSuffixSingular	-> Point
				ScoreFormatSuffixPlural		-> Points
				ScoreRange					-> {0, 100000}
				SortOrder					-> HighToLow
				
			Item 1 (dictionary)
			  .
			  .
			  .
			  .

Now pass the 2 arrays defined inside the plist into the first and second argument of `GameKitCenter`'s `initWithAchievements:leaderboards:andViewController:` method:

	// I'm using CCFileUtils here, you can use whatever you like to
	// get the file
	NSDictionary *gameKitCenterDictionary = [NSDictionary dictionaryWithContentsOfFile:[[CCFileUtils sharedFileUtils] fullPathFromRelativePath:@"GameKitCenter.plist"]];

	GameKitCenter *gkCenter = [[GameKitCenter alloc] initWithAchievements:gameKitCenterDictionary[@"Achievements"] andLeaderboards:gameKitCenterDictionary[@"Leaderboards"] nil];
	

The third argument is ignored for now, and will be explained later. If you want to listen to events generated by GameKitCenter, have your class implement `GameKitCenterDelegate` protocol, and add the class as a delegate:

	[gkCenter addDelegate:yourClassInstance];


Now, if your app is Game Center enabled, call `authenticateLocalPlayer` to connect to Game Center.

	[gkCenter authenticateLocalPlayer];
	
#####Reporting Achievement Progress

Reporting achievement progress is done by calling `reportAchievementWithIdentifier:percentageCompleted:`.

	[gkCenter reportAchievementWithIdentifier:@"Achievement_1" percentageCompleted:50.0];
	
However, note that this progress will **not be reported immediately** to the Game Center. It will be stored inside a queue. To have the queued achievements reported, call `reportQueuedAchievements`.

	[gkCenter reportQueuedAchievements];
	
The reason is of course, to avoid having to connect to Game Center every now and then, which might slow down your app. For example, if you have an achievement like "Kill 1,000,000 enemies", you can call `reportAchievementWithIdentifier:percentageCompleted:` after every enemy kill, and only call `reportQueuedAchievements` at the end of the level.

The queued achievements can also be removed by calling `flushQueuedAchievements`. It's useful if your game only saves achievements progress only if the player actually wins the level.

	[gkCenter flushQueuedAchievements];
	
#####Reset Achievements Progress

	[gkCenter resetAchievements];

#####Reporting Score

Reporting score is done by calling	`reportScore:leaderboardID`:

	[gkCenter reportScore:12345 leaderboardID:@"Leaderboard_1"];
	
#####(Important) Saving and Loading GameKitCenter

Apart from sending data to the Game Center, GameKitCenter also stores achievements and scores data locally. However, this data will be lost if you don't save it somewhere. The method `save` will return an `NSDictionary` that contains the entire state of GameKitCenter. You can choose to save this dictionary with the method of your choice (the sample project uses `NSMutableData`).

	NSDictionary *gkCenterState = [gkCenter save];
	// Save this somewhere
	
Loading the GameKitCenter state is done by calling `loadFromDictionary:`:

	NSDictionary *gkCenterState = /* get the saved state accordingly */;
	[gkCenter loadFromDictionary:gkCenterState];
	
#####Displaying the Standard View

For achievements:

	[gkCenter showAchievements];
	
For leaderboard:

	[gkCenter showLeaderboard:@"Leaderboard_1"];
	
	
###Advanced: Implementing Custom Classes

To do.